from fastapi import APIRouter, Depends, Query
from sqlmodel import Session, select
from backend.database import get_session
from backend.models.models import Shop, Product, Category
from typing import List, Union

router = APIRouter()


@router.get("/", response_model=List[Union[Shop, Product]])
def search(
    name: str = Query(None, description="Name to search"),
    category: str = Query(None, description="Category to filter by (for products)"),
    search_type: str = Query(
        "both", description="Search type: 'shops', 'products', or 'both'"
    ),
    db: Session = Depends(get_session),
):
    results = []

    if search_type in ["shops", "both"]:
        shop_query = select(Shop)
        if name:
            shop_query = shop_query.where(Shop.name.ilike(f"%{name}%"))
        results.extend(db.exec(shop_query).all())

    if search_type in ["products", "both"]:
        product_query = (
            select(Product).join(Category).where(True)
        )  # Ensures valid syntax
        if name:
            product_query = product_query.where(Product.name.ilike(f"%{name}%"))
        if category:
            product_query = product_query.where(Category.name.ilike(f"%{category}%"))
        results.extend(db.exec(product_query).all())

    return results
